/*
 *  Copyright (C) 2011
 *   Masaryk University, Faculty of Informatics
 *   Brno University of Technology, Faculty of Information Technology
 *
 *  Author(s): Daniel Kopecek <xkopecek@fi.muni.cz>
 *             Matej Lexa <lexa@fi.muni.cz>
 *             Tomas Martinek <martinto@fit.vutbr.cz>
 *
 *  LICENSE TERMS
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *  3. Neither the name of the Company nor the names of its contributors
 *  may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  This software is provided ``as is'', and any express or implied
 *  warranties, including, but not limited to, the implied warranties of
 *  merchantability and fitness for a particular purpose are disclaimed.
 *  In no event shall the company or contributors be liable for any
 *  direct, indirect, incidental, special, exemplary, or consequential
 *  damages (including, but not limited to, procurement of substitute
 *  goods or services; loss of use, data, or profits; or business
 *  interruption) however caused and on any theory of liability, whether
 *  in contract, strict liability, or tort (including negligence or
 *  otherwise) arising in any way out of the use of this software, even
 *  if advised of the possibility of such damage.
 */
#ifndef LIBTRIPLEX_H
#define LIBTRIPLEX_H

#include <stdarg.h>
#include "gff3.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * All API functions returning an `int' type return 0 on success.
 * The LIBTRIPLEX_ESYSTEM code means that the real reason/code should
 * be available in the global variable `errno'.
 */
#define LIBTRIPLEX_ESYSTEM  1 /**< System or (g)libc call failure */
#define LIBTRIPLEX_EINVALP  2 /**< Unknown/Invalid parameter */
#define LIBTRIPLEX_EINVALV  3 /**< Invalid value */
#define LIBTRIPLEX_ECUTOFF  4 /**< Cutoff treshold reached */
#define LIBTRIPLEX_ESOURCE  5 /**< A problem with the data source */
#define LIBTRIPLEX_EFAULT   6 /**< NULL parameter where a non-NULL is required */
#define LIBTRIPLEX_ENOMEM   7 /**< No memory */

        /**
         * Opaque type representing a triplex search context.
         */
        typedef struct triplexCTX triplexCTX;

        /**
         * Allocate and initialize a new context.
         */
        triplexCTX *libtriplex_init(void);

#define LIBTRIPLEX_FLAG_CDSOFF 0x00000001 /**< turn off coding segment mapping */
#define LIBTRIPLEX_FLAG_IDXOFF 0x00000002 /**< don't use FASTA indexing */

        /**
         * Set context flags.
         */
        int libtriplex_setflags(triplexCTX *ctx, uint32_t flags);

        /**
         * Unset context flags.
         */
        int libtriplex_unsetflags(triplexCTX *ctx, uint32_t flags);

        /**
         * Free all resources used by a context.
         */
        void libtriplex_fini(triplexCTX *ctx);

        /**
         * value types
         */
#define TRIPLEX_PARAM_INT      0x00000001
#define TRIPLEX_PARAM_FLOAT    0x00000002
#define TRIPLEX_PARAM_STRING   0x00000003
#define TRIPLEX_PFLAG_ARRAY    0x00000010

        /**
         * Set or unset a parameter.
         */
        int libtriplex_setparam(triplexCTX *ctx, const char *name, uint32_t type, ...);

        /**
         * Set or unset a parameter using a string representation of the value.
         * The function does the casting to the right type for us.
         */
        int libtriplex_setparam_coerce(triplexCTX *ctx, const char *name, const char *value);

        /**
         * Get the value of a context parameter.
         */
        int libtriplex_getparam(triplexCTX *ctx, const char *name, uint32_t type, ...);

        /**
         * Start triplex search.
         */
        int libtriplex_search(triplexCTX *ctx, const char *path, gff3_t *result);

#ifdef __cplusplus
}
#endif
#endif /* LIBTRIPLEX_H */
